package pencilbox.hakyukoka;

import java.util.HashSet;

import pencilbox.common.core.Address;
import pencilbox.common.core.Area;
import pencilboxex.SolverBase;
import solver.SolverConst;
import solver.rippleeffect.Answer;
import solver.rippleeffect.Entry;
import solver.rippleeffect.Cell;
import solver.rippleeffect.SolverImpl;
import solver.rippleeffect.SolveState;

public class Solver extends SolverBase<Board> {
	
	private Answer answer = null;
	
	/**
	 * 
	 */
	@Override
	public int solveProblem(Board board) throws InterruptedException {
		
		int width = board.cols();
		int height = board.rows();
		
		SolverImpl solverImpl = new SolverImpl();
		SolveState solveState = new SolveState(width, height);
		
		answer = null;
		
		for (Area area : board.getAreaList()) {
			Cell[] cells = new Cell[area.size()];
			int index = 0;
			for (Address addr : area) {
				cells[index++] = solveState.getCell(addr.c(), addr.r());
			}
			solveState.addArea(cells);
		}
		
		for (int y = 0; y < height; ++y) {
			for (int x = 0; x < width; ++x) {
				int state = board.getState(y, x);
				if (state > 0) {
					int num = board.getNumber(y, x);
					if (num > 0) {
						if (!solveState.setNumber(x, y, num)) {
							return SolverConst.INPUTERROR;
						}
					}
				}
			}
		}
		solveState.prep();
		
		int result = solverImpl.start(solveState);
		
		if (result != SolverConst.CANCEL || result != SolverConst.NORESULT) {
			answer = solveState.getAnswer();
		}
		
		return result;
	}

	/**
	 *
	 */
	@Override
	public void updateBoard(Board board) {

		if (board != null) {
			board.clearBoard();
			
			if (answer != null) {
				int size = answer.historyCount();
				
				for (int i = 0; i < size; ++i) {
					Entry entry = answer.getHistory(i);
					if (entry.number > 0) {
						board.enterNumberA(entry.y, entry.x, entry.number);
					}
				}
			}
		}
	}
	
	@Override
	protected boolean precheck(Board board) {
	
		boolean isSepatrate = false;
		HashSet<Address> set = new HashSet<Address>();
		
		
		for (Area area : board.getAreaList()) {
			int size = area.size();
			if (size >= 2) {
				set.clear();
				Address base = area.iterator().next();
				set.add(base);
				grouping(set, area, base);
				if (size != set.size()) {
					isSepatrate = true;
					break;
				}
			}
		}
		
		if (isSepatrate) {
			return confirm("ђnɂȂĂ镔܂B\o[N܂H");
		}
		
		return true;
	}

	private void grouping(HashSet<Address> set, Area area, Address base) {
		
		int r = base.r();
		int c = base.c();
		
		for (Address addr : area) {
			if (!set.contains(addr)) {
				if (Math.abs(r - addr.r()) + Math.abs(c - addr.c()) == 1) {
					set.add(addr);
					grouping(set, area, addr);
				}
			}
		}
	}
	
}
