package pencilboxex;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;


import pencilbox.common.core.BoardBase;
import pencilbox.common.gui.Frame;
import solver.SolverConst;

/**
 * \o[NA~ʕ\ȂǂT|[g_CAO
 */
public final class SolverDialog<T extends BoardBase> extends JDialog implements ActionListener, Runnable {
	
	/** ^Cg */
	private static final String TITLE = "\o[";
	
	/** _CAO̐et[ */
	private Frame frame;
	/** x */
	private JLabel label = new JLabel("𓚒");
	/** bZ[Wx */
	private JLabel messageLabel = new JLabel(" ");
	/** {^ */
	private JButton button = new JButton("~");
	/** G[bZ[W */
	private String errMessage = "";
	
	/** Xbh */
	private Thread thread = null;
	/** \o[ */
	private SolverBase<T> solver = null;
	/** Ֆ */
	private T board = null;
	/**  */
	private int result = SolverConst.CANCEL;
	
	/**
	 * \o[N̈A̗s
	 * 
	 * @param frame _CAO̐et[
	 * @param solver \o[
	 * @param board Ֆ
	 * @return 
	 */
	public static <T extends BoardBase> int start(final Frame frame, final SolverBase<T> solver, final T board) {
		SolverDialog<T> dialog = new SolverDialog<T>(frame, solver, board);
		
		solver.setDialog(dialog);
		
		dialog.pack();
		dialog.setLocationRelativeTo(frame);
		
		dialog.start();
		
		if (dialog.thread != null) {
			dialog.setVisible(true);
		}
		
		//
		int result = dialog.result;
		
		switch (result) {
		case SolverConst.COMPLATE:
			JOptionPane.showMessageDialog(frame, "B݂܂", TITLE, JOptionPane.INFORMATION_MESSAGE);
			break;
		case SolverConst.INPUTERROR:
			JOptionPane.showMessageDialog(frame, "̓G[F" + dialog.errMessage, TITLE, JOptionPane.ERROR_MESSAGE);
			break;
		case SolverConst.NORESULT:
			JOptionPane.showMessageDialog(frame, "݂܂", TITLE, JOptionPane.WARNING_MESSAGE);
			break;
		case SolverConst.MULTI:
			JOptionPane.showMessageDialog(frame, "݂܂", TITLE, JOptionPane.WARNING_MESSAGE);
			break;
		case SolverConst.UNSOLVED:
			JOptionPane.showMessageDialog(frame, "܂ł", TITLE, JOptionPane.INFORMATION_MESSAGE);
			break;
		}
		
		return result;
	}
	
	/**
	 * RXgN^
	 * 
	 * @param frame _CAO̐et[
	 * @param solver \o[
	 * @param board Ֆ
	 */
	private SolverDialog(Frame frame, SolverBase<T> solver, T board) {
		super(frame, "\o[", true);
		this.frame = frame;
		this.solver = solver;
		this.board = board;
		
		getContentPane().setLayout(new BorderLayout());
		JPanel panel = new JPanel(new BorderLayout());
		JPanel labelPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		
		panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		messageLabel.setBorder(BorderFactory.createLoweredBevelBorder());
		
		labelPanel.add(label);
		panel.add(labelPanel, BorderLayout.CENTER);
		buttonPanel.add(button);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(panel, BorderLayout.CENTER);
		getContentPane().add(messageLabel, BorderLayout.SOUTH);
		
		button.addActionListener(this);
	}
	
	/**
	 * XbhN
	 */
	public void start() {
		
		thread = new Thread(this);
		
		thread.start();
	}
	
	/**
	 * \o[NAIʂ
	 */
	public void run() {
		
		//lLZƂ
		result = SolverConst.CANCEL;
		try {
			if (solver.precheck(board)) {
				long time = System.nanoTime();
				result = solver.solveProblem(board);
				System.out.printf("%,19dns\n", System.nanoTime() - time);
				try {
					SwingUtilities.invokeAndWait(new Runnable() {
						public void run() {
							solver.updateBoard(board);
						}
					});
				} catch (InvocationTargetException e) {
				}
				frame.repaint();
			}
		} catch (InterruptedException e) {
			//ꉞI
			result = SolverConst.CANCEL;
		} catch (RuntimeException e) {
			e.printStackTrace();
			setErrMessage(e.toString());
			result = SolverConst.INPUTERROR;
		} finally {
			synchronized (this) {
				thread = null;
				//ʂ
				setVisible(false);
			}
		}
	}

	/**
	 * ANVCxg
	 * 
	 * @param e ANVCxg
	 */
	public void actionPerformed(ActionEvent e) {

		if (e.getSource() == button) {
			synchronized (this) {
				if (thread != null) {
					thread.interrupt();
				}
			}
		}
	}
	
	/**
	 * bZ[Wݒ肷
	 * 
	 * @param message bZ[W
	 */
	public void setMessage(String message) {
		messageLabel.setText(message);
	}
	
	/**
	 * G[bZ[W擾
	 * 
	 * @return G[bZ[W
	 */
	public String getErrMessage() {
		return errMessage;
	}

	/**
	 * G[bZ[Wݒ肷
	 * 
	 * @param errMessage G[bZ[W
	 */
	public void setErrMessage(String errMessage) {
		this.errMessage = errMessage;
	}

	public boolean confirm(String message) {
		return JOptionPane.showConfirmDialog(frame, message, TITLE, JOptionPane.YES_NO_OPTION) == JOptionPane.OK_OPTION;
	}
}
