package solver.rippleeffect;

public class Cell {
	
	public final int x;
	public final int y;
	
	int number = 0;
	int flag = -1;
	Area area = null;
	boolean changed = false;
	
	public Cell(int x, int y) {
		this.x = x;
		this.y = y;
	}
	
	public boolean off(int flag, int num) {
		if ((this.flag & flag) != 0) {
			this.flag &= ~flag;
			changed = true;
			if (num > 0 && area != null) {
				int count = area.count[num - 1];
				if (count > 0) {
					area.count[num - 1] = count - 1;
				}
				area.changed = true;
			}
			return true;
		}
		return false;
	}

	public Entry createHistoryEntry() {
		return new Entry(x, y, number, flag);
	}

	public void set(Cell cell) {
		number = cell.number;
		flag = cell.flag;
		changed = cell.changed;
	}
	
	@Override
	public String toString() {
		return "x=" + x + ", y=" + y + ", number=" + number + ", flag=" + flag;
	}

	@Override
	public int hashCode() {

		final int prime = 31;
		int result = 1;
		result = prime * result + flag;
		result = prime * result + number;
		result = prime * result + x;
		result = prime * result + y;
		return result;
	}

	@Override
	public boolean equals(Object obj) {

		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		final Cell other = (Cell)obj;
		if (flag != other.flag) return false;
		if (number != other.number) return false;
		if (x != other.x) return false;
		if (y != other.y) return false;
		return true;
	}
}
