package solver.rippleeffect;

import java.util.ArrayList;

import solver.State;

/**
 * \o[pĂՖʂ̏
 */
public class SolveState extends State<Entry> {
	
	/** Ֆʂ̕ */
	public final int width;
	/** Ֆʂ̍ */
	public final int height;
	/** Ֆ */
	final Cell[][] board;
	/** GA */
	final ArrayList<Area> areas = new ArrayList<Area>();
	/**  */
	final ArrayList<Entry> history = new ArrayList<Entry>();
	
	/** ̃|C^ */
	private Pointer pointer = null;
	/** CfbNX̏d݂炩ߎZo */
	int[] indexes;
	
	/**
	 * RXgN^
	 * 
	 * @param width 
	 * @param height 
	 */
	public SolveState(int width, int height) {
		this.width = width;
		this.height = height;
		this.board = new Cell[height][width];
		
		for (int y = 0; y < height; ++y) {
			Cell[] array = board[y];
			for (int x = 0; x < width; ++x) {
				array[x] = new Cell(x, y);
			}
		}
	}
	
	public SolveState(SolveState parent) {
		this.width = parent.width;
		this.height = parent.height;
		board = new Cell[height][width];
		indexes = parent.indexes;
		
		for (int y = 0; y < height; ++y) {
			Cell[] array = board[y];
			for (int x = 0; x < width; ++x) {
				array[x] = new Cell(x, y);
			}
		}
		
		ArrayList<Area> parentAreas = parent.areas;
		int size = parentAreas.size();
		
		for (int i = 0; i < size; ++i) {
			Area area = parentAreas.get(i);
			Cell[] cells = area.cells;
			Cell[] newCells = new Cell[cells.length];
			Area newArea = new Area(newCells);
							
			for (int j = 0; j < cells.length; ++j) {
				Cell cell = cells[j];
				Cell newCell = board[cell.y][cell.x];
				newCell.area = newArea;
				newCells[j] = newCell;
			}
			areas.add(newArea);
		}
		
		set(parent);
	}
	
	public void prep() {
		//ׂẴGA𐸍AȅdvxZo
		//܂͕̍ő}XZo
		int maxSize = 0;
		
		for (Area area : areas) {
			int size = area.cells.length;
			if (size > maxSize) {
				maxSize = size;
			}
		}
		//dZo
		//~igp񐔁|Pj
		int[] waits = new int[maxSize];
		indexes = new int[maxSize];
		for (int i = 0; i < maxSize; ++i) {
			waits[i] = - i - 1;
		}
		for (Area area : areas) {
			int size = area.cells.length;
			for (int i = 0; i < size; ++i) {
				waits[i] += (i + 1);
			}
		}
		//̏Ԃ擾
		for (int i = 0; i < maxSize; ++i) {
			int wait = waits[i];
			int count = 0;
			for (int j = 0; j < maxSize; ++j) {
				if (waits[j] > wait) {
					++count;
				}
			}
			while (indexes[count] != 0) {
				++count;
			}
			indexes[count] = i + 1;
		}
	}
	
	/**
	 * Z擾
	 * 
	 * @param x xW
	 * @param y yW
	 * @return Z
	 */
	public Cell getCell(int x, int y) {
		return board[y][x];
	}
	
	/**
	 * GAǉ
	 * (Ɏgp)
	 * 
	 * @param cells Z̔z
	 */
	public void addArea(Cell[] cells) {
		
		int flag = (1 << cells.length) - 1;
		Area area = new Area(cells);
		
		for (int i = 0; i < cells.length; ++i) {
			Cell cell = cells[i];
			
			cell.area = area;
			cell.flag = flag;
			cell.changed = true;
		}
		
		areas.add(area);
	}

	public boolean setNumber(int x, int y, int num) {
		//cɒlݒ
		//tO쐬
		int flag = (1 << (num - 1));
		Cell cell = board[y][x];
		
		if ((cell.flag & flag) == 0) {
			return false;
		}
		
		int oldFlag = cell.flag;
		cell.flag = flag;
		cell.number = num;
		cell.changed = false;
		
		history.add(cell.createHistoryEntry());
		
		Area area = cell.area;
		if (area != null) {
			int[] count = area.count;
			for (int i = 0; i < count.length; ++i) {
				if (i == num - 1) {
					count[i] = -1;
				} else {
					if ((oldFlag & (1 << i)) != 0) {
						--count[i];
					}
				}
			}
			Cell[] cells = area.cells;
			for (int i = 0; i < cells.length; ++i) {
				if (cells[i] != cell) {
					cells[i].off(flag, num);
				}
			}
			area.changed = true;
		}
		
		int leftX = x - num;
		if (leftX < 0 ) leftX = 0;
		for (; leftX < x; ++leftX) {
			board[y][leftX].off(flag, num);
		}
		int rightX = x + num;
		if (rightX >= width ) rightX = width - 1;
		for (; rightX > x; --rightX) {
			board[y][rightX].off(flag, num);
		}
		int upY = y - num;
		if (upY < 0 ) upY = 0;
		for (; upY < y; ++upY) {
			board[upY][x].off(flag, num);
		}
		int downY = y + num;
		if (downY >= height ) downY = height - 1;
		
		for (; downY > y; --downY) {
			board[downY][x].off(flag, num);
		}
		
		return true;
	}
	
	
	public Answer getAnswer() {
		Answer answer = new Answer(width, height);
		
		for (int y = 0; y < height; ++y) {
			Cell[] array = board[y];
			for (int x = 0; x < width; ++x) {
				answer.setEntry(x, y, array[x].createHistoryEntry());
			}
		}
		
		answer.setHistory(history.toArray(new Entry[0]));
		
		return answer;
	}
	
	/**
	 * 
	 * @return
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		
		StringBuffer buffer = new StringBuffer();
		StringBuffer buffer2 = new StringBuffer();
		StringBuffer buffer3 = new StringBuffer();
		
		for (int y = 0; y < board.length; ++y) {
			Cell[] array = board[y];
			for (int x = 0; x < array.length; ++x) {
				Cell cell = array[x];
				if (cell.number > 0) {
					buffer.append(cell.number);
				} else {
					buffer.append(' ');
				}
				buffer2.append(Integer.toHexString(cell.flag));
				buffer2.append('\t');
				if (cell.changed) {
					buffer3.append('*');
				} else {
					buffer3.append(' ');
				}
			}
			buffer.append('\n');
			buffer2.append('\n');
			buffer3.append('\n');
		}
		buffer.append('\n');
		buffer.append(buffer2);
		buffer.append('\n');
		buffer.append(buffer3);
		
		for (Area area : areas) {
			buffer.append(area);
			buffer.append('\n');
		}
		
		return buffer.toString();
	}

	public Pointer getPointer() {
		
		if (pointer == null) {
			pointer = new Pointer(this);
		}
		
		pointer.init();
		
		return pointer;
	}

	public SolveState copy() {
		return new SolveState(this);
	}

	public void set(SolveState state) {
		Cell[][] srcBoard = state.board;
		ArrayList<Area> srcAreas = state.areas;
		
		for (int y = 0; y < height; ++y) {
			Cell[] cells = board[y];
			Cell[] srcCells = srcBoard[y];
			for (int x = 0; x < width; ++x) {
				cells[x].set(srcCells[x]);
			}
		}
		int size = srcAreas.size();
		for (int i = 0; i < size; ++i) {
			areas.get(i).set(srcAreas.get(i));
		}
		history.clear();
		history.addAll(state.history);
	}

	@Override
	public void apply(Entry entry) {
		setNumber(entry.x, entry.y, entry.number);
	}
	
	@Override
	public boolean deny(Entry entry) {
		return board[entry.y][entry.x].off(entry.flag, entry.number);
	}

	@Override
	public int hashCode() {
		int code = 0;
		for (int y = 0; y < height; ++y) {
			Cell[] cells = board[y];
			for (int x = 0; x < width; ++x) {
				code += cells[x].number << (x + y);
			}
		}
		return code;
	}

	@Override
	public boolean equals(Object obj) {

		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		final SolveState other = (SolveState)obj;
		
		Cell[][] otherBoard = other.board;
		if (width != other.width) return false;
		if (height != other.height) return false;
		for (int y = 0; y < height; ++y) {
			Cell[] cells = board[y];
			Cell[] otherCells = otherBoard[y];
			for (int x = 0; x < width; ++x) {
				if (cells[x].number != otherCells[x].number) {
					return false;
				}
			}
		}
		
		return true;
	}
	
	
}
